import { createEffect, createRoot, createSignal, Signal } from 'solid-js';
import { watchDb } from '../db-admin/health';

declare global {
  var importQueue: Map<string, any>;
}

globalThis.importQueue = new Map<string, Signal<any>>();

const providers = {
  watchDb,
};

onmessage = function (e) {
  createRoot((dispose) => {
    const { provider, params, initialValue, _release, _id } = e.data;
    console.log({
      caller: 'solid-worker / onmessage',
      e,
      provider,
      params,
      initialValue,
      _release,
      _id,
    });

    if (_release) {
      dispose();
      return;
    }

    if (!Object.hasOwn(providers, provider)) {
      console.error({
        caller: 'solid-worker',
        message: 'Unknown provider',
        provider,
      });
    }

    const signal = createSignal(initialValue);
    //    globalThis.importQueue.set(_id, signal);  // Would be needed for bidirectional signals...

    createEffect(() => {
      console.log({
        caller: 'solid_worker / effect',
        provider,
        params,
        initialValue,
        _release,
        currentValue: signal[0](),
        _id,
      });
      postMessage({ _id, _updatedValue: signal[0]() });
    });

    const providerFunction = providers[provider as keyof typeof providers];
    providerFunction({ params, signal });
  });
};
